// $Id: RETestControl.java,v 1.3 2006/07/08 21:57:39 sjakob Exp $
/*
 *  RETestControl.java - The controller class which responds to events
 *  generated by a RETestView object. For use in the RETest plugin for jEdit.
 *  Copyright (C) 2000 Steve Jakob
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.awt.event.*;
import java.util.regex.*;
import javax.swing.*;
import org.gjt.sp.jedit.*;
/**
 * The class is designed for use with jEdit for testing regular expressions as
 * per the gnu.regexp implementation included with jEdit. It responds to events
 * generated by the view object, RETestView, and performs the actual tests.
 *
 * @author   Steve Jakob
 */
public class RETestControl
implements ActionListener, ItemListener, FocusListener
{
	private RETestView myView;
	private StatusPane sp;
	private View jeditView = null;
	private static Pattern slashRE = null;

	//Constructors
	public RETestControl()
	{
		super();
		initialize();
	}

	/**
	 * Establish the jEdit view for which this object was created. It is required
	 * to allow the RE string to be inserted into the current buffer.
	 *
	 * @param vw  The current jEdit view.
	 */
	public void setJeditView(View vw)
	{
		jeditView = vw;
	}

	//Implementors
	/**
	 * Perform any required initialization of the RETestControl object. Create the
	 * view object.
	 */
	private void initialize()
	{
		myView = new RETestView(this);
		sp = myView.getStatusField();
		myView.pack();
	}

	/**
	 * Display the RETestView object.
	 */
	public void displayView()
	{
		myView.setVisible(true);
	}

	/**
	 * Respond to the selection of one of the buttons in the button panel.
	 *
	 * @param evt  The ActionEvent generated by the button press.
	 */
	public void actionPerformed(ActionEvent evt)
	{
		String btnPressed = ((JButton) evt.getSource()).getActionCommand();
		if (btnPressed.equals("Test"))
		{
			try
			{
				Pattern testRE = Pattern.compile(myView.getREString());
				String testString = myView.getTestString();
				String replaceString = myView.getReplaceString();
				String selection = myView.getSelectedFunction();
				if (selection.equals("matches"))
				{
					testIsMatch(testRE, testString);
				}
				if (selection.equals("find"))
				{
					testGetMatch(testRE, testString);
				}
				if (selection.equals("find (all matches)"))
				{
					testGetAllMatches(testRE, testString);
				}
				if (selection.equals("replaceFirst"))
				{
					testSubstitute(testRE, testString, replaceString);
				}
				if (selection.equals("replaceAll"))
				{
					testSubstituteAll(testRE, testString, replaceString);
				}
			}
			catch (PatternSyntaxException pse)
			{
				sp.setMsg("ERROR: Bad regular expression.");
			}
		}
		if (btnPressed.equals("Close"))
		{
			myView.setVisible(false);
		}
		if (btnPressed.equals("Insert"))
		{
			if (slashRE == null)
			{
				try
				{
					slashRE = Pattern.compile("\\\\");
				}
				catch (PatternSyntaxException e)
				{
					sp.setMsg("RETestContol: Error creating slashRE.");
					System.out.println(e);
				}
			} else
			{
				Matcher slashMatcher = slashRE.matcher(myView.getREString());
				jeditView.getTextArea().setSelectedText(
						slashMatcher.replaceAll("\\\\"));
			}
		}
		if (btnPressed.equals("Help"))
		{
			myView.toggleHelpPanel();
		}
	}

	/**
	 * Respond to the selection of one of the functions in the function list.
	 *
	 * @param e    The ItemEvent generated by the selection.
	 */
	public void itemStateChanged(ItemEvent e)
	{
		if (e.getStateChange() == ItemEvent.SELECTED)
		{
			myView.clearStatusMessage();
			String selection = myView.getSelectedFunction();
			if (selection.equals("replaceFirst") ||
					selection.equals("replaceAll"))
			{
				myView.setReplaceFieldEnabled(true);
			} else
			{
				myView.setReplaceFieldEnabled(false);
			}
		}
	}
	
	public void focusGained(FocusEvent e)
	{
		myView.clearStatusMessage();
	}
	
	public void focusLost(FocusEvent e)
	{
		// Do nothing
	}

	/**
	 * Test the entered regular expression against the entered test string. Set the
	 * status message field to indicate success or failure.
	 *
	 * @param testRE      The regular expression to be tested.
	 * @param testString  The string against which the test is run.
	 */
	private void testIsMatch(Pattern testRE, String testString)
	{
		Matcher matcher = testRE.matcher(testString);
		if (matcher.matches())
		{
			sp.setSuccessMsg("We have a match!!!");
		} else
		{
			sp.setFailureMsg("Sorry, try again.");
		}
	}

	/**
	 * Test the entered regular expression against the entered test string. If a
	 * match exists, copy the test string to the status message field, highlighting
	 * the match.
	 *
	 * @param testRE      The regular expression to be tested.
	 * @param testString  The string against which the test is run.
	 */
	private void testGetMatch(Pattern testRE, String testString)
	{
		Matcher matcher = testRE.matcher(testString);
		if (matcher.find())
		{
			sp.setMsg(testString);
			sp.addHighlight(matcher.start(), matcher.end());
		} else
		{
			sp.setFailureMsg("Sorry, try again.");
		}
	}

	/**
	 * Test the entered regular expression against the entered test string. If a
	 * match exists, copy the test string to the status message field, highlighting
	 * all matches.
	 *
	 * @param testRE      The regular expression to be tested.
	 * @param testString  The string against which the test is run.
	 */
	private void testGetAllMatches(Pattern testRE, String testString)
	{
		Matcher matcher = testRE.matcher(testString);
		if (matcher.find())
		{
			sp.setMsg(testString);
			sp.addHighlight(matcher.start(), matcher.end());
			while (matcher.find())
			{
				sp.addHighlight(matcher.start(), matcher.end());
			}
		} else
		{
			sp.setFailureMsg("Sorry, try again.");
		}
	}

	/**
	 * Replace the first occurrence of the RE in the test string with the replace
	 * string. Display the result in the status message field.
	 *
	 * @param testRE         The regular expression to be tested.
	 * @param testString     The string against which the test is run.
	 * @param replaceString  The string used to replace matches in the test string.
	 */
	private void testSubstitute(Pattern testRE, String testString, String replaceString)
	{
		Matcher matcher = testRE.matcher(testString);
		sp.setMsg(matcher.replaceFirst(replaceString));
	}

	/**
	 * Replace all occurrences of the RE in the test string with the replace
	 * string. Display the result in the status message field.
	 *
	 * @param testRE         The regular expression to be tested.
	 * @param testString     The string against which the test is run.
	 * @param replaceString  The string used to replace matches in the test string.
	 */
	private void testSubstituteAll(Pattern testRE, String testString, String replaceString)
	{
		Matcher matcher = testRE.matcher(testString);
		sp.setMsg(matcher.replaceAll(replaceString));
	}

}

