// $Id: RETestView.java,v 1.3 2006/07/08 21:57:39 sjakob Exp $
/*
 *  RETestView.java - The user interface class for a jEdit plugin which
 *  allows the user to test regular expressions as per the gnu.regexp
 *  implementation.
 *  Copyright (C) 2000 Steve Jakob
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import org.gjt.sp.jedit.*;
/**
 * The class is designed for use with jEdit for testing regular expressions as
 * per the gnu.regexp implementation included with jEdit. The user enters a
 * regular expression and a string to test against the RE, clicks on the "Test"
 * button, and is given an indication of whether or not the entered string would
 * be matched by the RE.
 *
 * @author   Steve Jakob
 */
public class RETestView extends JFrame
{
	private JLabel viewTitle;
	private JLabel functionPrompt;
	private JLabel rePrompt;
	private JLabel stringPrompt;
	private JLabel replacePrompt;
	private FunctionList functionList = new FunctionList();
	private JTextField reField = new JTextField();
	private JTextField stringField = new JTextField();
	private JTextField replaceField = new JTextField();
	private StatusPane statusField = new StatusPane();
	// private JTextField statusField = new JTextField();
	private JButton testButton = new JButton("Test");
	private JButton closeButton = new JButton("Close");
	private JButton insertButton = new JButton("Insert");
	private JButton helpButton = new JButton("Help");
	private HelpPanel helpPanel;
	private RETestControl myController;

	//Constructors
	public RETestView(RETestControl controller)
	{
		super("RETest");
		myController = controller;
		loadStrings();
		init();
	}

	/**
	 * Returns the index string of the selected item in the function list. <BR>
	 * This could be one of: isMatch, getMatch, getMatches, substitute,
	 * substituteAll, or getMatchEnumeration.
	 *
	 * @return   The selected function's index string.
	 */
	public String getSelectedFunction()
	{
		return FunctionList.getSelection(functionList.getSelectedIndex());
	}

	/**
	 * Enables or disables the "Replace String" field.
	 *
	 * @param enabled  True to enable, false to disable.
	 */
	public void setReplaceFieldEnabled(boolean enabled)
	{
		if (enabled != replaceField.isEnabled())
		{
			replaceField.setEnabled(enabled);
		}
	}

	public String getREString()
	{
		return reField.getText();
	}

	public String getTestString()
	{
		return stringField.getText();
	}

	public String getReplaceString()
	{
		return replaceField.getText();
	}

	public StatusPane getStatusField()
	{
		return statusField;
	}

	/**
	 * Displays a text string in the status message field.
	 *
	 * @param msg  Message to display.
	 */
	public void setStatusMessage(String msg)
	{
		statusField.setText(msg);
	}

	/**
	 * Clears the status message field.
	 */
	public void clearStatusMessage()
	{
		statusField.setText(null);
	}

	//Implementors
	/**
	 * Attempt to load title/prompt strings for the view from jEdit properties. If
	 * the properties are not defined, assign values.
	 */
	private void loadStrings()
	{
		String temp;
		temp = jEdit.getProperty("plugin.RETestPlugin.view.title");
		if (temp == null)
		{
			temp = "Regular Expression Tester";
		}
		viewTitle = new JLabel(temp, SwingConstants.CENTER);
		temp = jEdit.getProperty("plugin.RETestPlugin.function.prompt");
		if (temp == null)
		{
			temp = "Test function: ";
		}
		functionPrompt = new JLabel(temp, SwingConstants.RIGHT);
		temp = jEdit.getProperty("plugin.RETestPlugin.re.prompt");
		if (temp == null)
		{
			temp = "RE: ";
		}
		rePrompt = new JLabel(temp, SwingConstants.RIGHT);
		temp = jEdit.getProperty("plugin.RETestPlugin.teststring.prompt");
		if (temp == null)
		{
			temp = "Test String: ";
		}
		stringPrompt = new JLabel(temp, SwingConstants.RIGHT);
		temp = jEdit.getProperty("plugin.RETestPlugin.replacestring.prompt");
		if (temp == null)
		{
			temp = "Replace String: ";
		}
		replacePrompt = new JLabel(temp, SwingConstants.RIGHT);
	}

	/**
	 * Initialize the RE Test window GUI.
	 */
	private void init()
	{
		JPanel p = new JPanel();
		p.setLayout(new BorderLayout());
		p.setBorder(new EmptyBorder(5, 5, 5, 5));
		p.add(viewTitle, BorderLayout.NORTH);
		p.add(testPanel(), BorderLayout.CENTER);
		// add(statusField, BorderLayout.SOUTH);
		p.add(buttonPanel(), BorderLayout.SOUTH);
		helpPanel = new HelpPanel(RETestPlugin.class.getResource("syntax.html"));
		/*
		 *  JSplitPane mainPanel = new JSplitPane(JSplitPane.VERTICAL_SPLIT, p, helpPanel);
		 *  mainPanel.setDividerLocation((double)0.5);
		 */
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BorderLayout());
		mainPanel.add(p, BorderLayout.NORTH);
		mainPanel.add(helpPanel, BorderLayout.CENTER);
		setContentPane(mainPanel);

		replaceField.setEnabled(false);
	}

	/**
	 * Build the main section of the RE test window.
	 *
	 * @return   testPanel A JPanel containing the main section of the RE test
	 *      window.
	 */
	private JPanel testPanel()
	{
		GridBagLayout gb = new GridBagLayout();
		JPanel testPanel = new JPanel(gb);
		testPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.fill = GridBagConstraints.NONE;
		gbc.insets = new Insets(10, 10, 10, 10);
		// gbc.ipadx = 25;
		// gbc.ipady = 10;
		gbc.anchor = GridBagConstraints.EAST;
		gb.setConstraints(functionPrompt, gbc);
		testPanel.add(functionPrompt);
		gbc.gridy = 1;
		gb.setConstraints(rePrompt, gbc);
		testPanel.add(rePrompt);
		gbc.gridy = 2;
		gb.setConstraints(stringPrompt, gbc);
		testPanel.add(stringPrompt);
		gbc.gridy = 3;
		gb.setConstraints(replacePrompt, gbc);
		testPanel.add(replacePrompt);
		gbc.gridx = 1;
		gbc.gridy = 0;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.anchor = GridBagConstraints.CENTER;
		gbc.weightx = 1;
		gb.setConstraints(functionList, gbc);
		testPanel.add(functionList);
		// Controller listens for item selections/focus changes
		reField.addFocusListener(myController);
		stringField.addFocusListener(myController);
		replaceField.addFocusListener(myController);
		functionList.addItemListener(myController);
		gbc.gridy = 1;
		gb.setConstraints(reField, gbc);
		testPanel.add(reField);
		gbc.gridy = 2;
		gb.setConstraints(stringField, gbc);
		testPanel.add(stringField);
		gbc.gridy = 3;
		gb.setConstraints(replaceField, gbc);
		testPanel.add(replaceField);
		gbc.gridx = 0;
		gbc.gridy = 4;
		gbc.gridwidth = 2;
		gb.setConstraints(statusField, gbc);
		statusField.setEnabled(false);
		testPanel.add(statusField);
		return testPanel;
	}

	/**
	 * Create the button panel for the RE test GUI.
	 *
	 * @return   buttonPanel A JPanel containing the buttons for the RE test
	 *      window.
	 */
	private JPanel buttonPanel()
	{
		testButton = new JButton("Test");
		testButton.setActionCommand("Test");
		closeButton = new JButton("Close");
		closeButton.setActionCommand("Close");
		insertButton = new JButton("Insert");
		insertButton.setActionCommand("Insert");
		helpButton = new JButton("Help");
		helpButton.setActionCommand("Help");
		JPanel buttonPanel = new JPanel();
		buttonPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		JPanel p1 = new JPanel(new GridLayout(1, 4, 30, 5));
		p1.add(testButton);
		testButton.addActionListener(myController);
		p1.add(closeButton);
		closeButton.addActionListener(myController);
		p1.add(insertButton);
		insertButton.addActionListener(myController);
		p1.add(helpButton);
		helpButton.addActionListener(myController);
		// HelpPanel starts out displayed, so change button text
		helpButton.setText("Hide");
		buttonPanel.add(p1);
		return buttonPanel;
	}

	/**
	 * Toggles the display of the help panel. The help panel displays syntax and
	 * usage notes for the gnu.regexp package.
	 */
	public void toggleHelpPanel()
	{
		if (helpPanel.isVisible())
		{
			helpPanel.setVisible(false);
			helpButton.setText("Help");
		} else
		{
			helpPanel.setVisible(true);
			helpButton.setText("Hide");
		}
		this.pack();
	}

	private static class FunctionList extends JComboBox
	{

		private static String[] functions = {"matches", "find",
				"find (all matches)", "replaceFirst", "replaceAll"};

		public FunctionList()
		{
			super();
			this.setEditable(false);
			for (int i = 0; i < functions.length; i++)
			{
				this.addItem(getFunctionName(functions[i]));
			}
		}

		/**
		 * A convenience method to load the function names from the jEdit properties
		 * list. If the properties have not been defined, the String passed in as the
		 * index is used.
		 *
		 * @return       The function name corresponding to the supplied index.
		 * @param fname  The suffix for the properties index.
		 */
		private String getFunctionName(String fname)
		{
			String temp = jEdit.getProperty("plugin.RETestPlugin.flist." + fname);
			if (temp == null)
			{
				temp = fname;
			}
			return temp;
		}

		/**
		 * Converts the integer index of the function list selection to a more
		 * readable text string.
		 *
		 * @return       An index string corresponding to the selected function.
		 * @param index  The index of the selected function in the list.
		 */
		public static String getSelection(int index)
		{
			return functions[index];
		}

	}

}

